#
# spec file for package tvm
#
# Copyright (c) 2022 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%define _lto_cflags %{nil}

%{?!python_module:%define python_module() python3-%{**}}
%define skip_python2 1
# https://github.com/apache/tvm/issues/8577
%define skip_python39 1
%define skip_python310 1
%ifarch aarch64 x86_64 ppc64le
%bcond_without onednn
%else
%bcond_with onednn
%endif
%bcond_without pytest
%ifarch aarch64
%bcond_without arm_compute_lib
%else
%bcond_with arm_compute_lib
%endif
# regular cmake builddir conflicts with the python singlespec
%global __builddir build_cmake
Name:           tvm
Version:        0.8.0
Release:        bp156.5.7
Summary:        An end-to-end Deep Learning Compiler Stack
License:        Apache-2.0
URL:            https://tvm.apache.org/
Source:         https://github.com/apache/tvm/archive/v%{version}.tar.gz#/tvm-%{version}.tar.gz
# PATCH-FIX-UPSTREAM tvm-fix-relay-test.patch --  gh#apache/tvm#10402
Patch0:         tvm-fix-relay-test.patch
# PATCH-FIX-OPENSUSE lib-finder-python-cmake.patch -- custom cmake path
Patch1:         lib-finder-python-cmake.patch
# PATCH-FIX-OPENSUSE tvm-fix-openblas.patch -- We use openblas headers instead of netlib cblas
Patch2:         tvm-fix-openblas.patch
# PATCH-FIX-OPENSUSE tvm-disable-vulkan-test-check.patch -- Cannot test in OBS despite enabled in library
Patch3:         tvm-disable-vulkan-test-check.patch
# PATCH-FIX-OPENSUSE tvm-do-not-force-synr-version.patch -- boo#1197347
Patch4:         tvm-do-not-force-synr-version.patch
BuildRequires:  %{python_module Cython}
BuildRequires:  %{python_module attrs}
BuildRequires:  %{python_module cloudpickle}
BuildRequires:  %{python_module decorator}
BuildRequires:  %{python_module numpy}
BuildRequires:  %{python_module psutil}
BuildRequires:  %{python_module pytest}
BuildRequires:  %{python_module scipy}
BuildRequires:  %{python_module setuptools}
BuildRequires:  %{python_module synr}
BuildRequires:  %{python_module tornado}
%if %{with arm_compute_lib}
BuildRequires:  ComputeLibrary-devel
%endif
BuildRequires:  antlr4-java
BuildRequires:  cmake
BuildRequires:  dlpack-devel
BuildRequires:  dmlc-core-devel
BuildRequires:  fdupes
BuildRequires:  gcc-c++
BuildRequires:  git
BuildRequires:  gmock
BuildRequires:  gtest
%if 0%{?suse_version} > 1550 || ( 0%{?is_opensuse} && 0%{?sle_version} > 150400 )
# Cannot build with llvm15, so stick with llvm14 for now
BuildRequires:  llvm14-devel
%else
BuildRequires:  llvm-devel
%endif
BuildRequires:  memory-constraints
BuildRequires:  libopenblas_pthreads-devel
BuildRequires:  opencl-headers
BuildRequires:  pkgconfig
BuildRequires:  rang-devel
BuildRequires:  spirv-headers
BuildRequires:  spirv-tools
BuildRequires:  spirv-tools-devel
BuildRequires:  vulkan-devel
BuildRequires:  pkgconfig(OpenCL)
BuildRequires:  pkgconfig(gl)
BuildRequires:  pkgconfig(glfw3)
Requires:       python-attrs
Requires:       python-cloudpickle
Requires:       python-decorator
Requires:       python-numpy
Requires:       python-psutil
Requires:       python-synr
%if %{with onednn}
BuildRequires:  onednn-devel
%endif
# Tests are failing on 32-bit
ExcludeArch:    %{arm} %{ix86}
%python_subpackages

%description
TVM is a deep learning compiler stack for CPUs, GPUs, and specialized accelerators.

%package -n tvmc
Summary:        TVM command line driver
Requires:       libtvm = %{version}
%if 0%{?suse_version} > 1550
# Tumbleweed defaults to python 3.10 which is not compatible yet
Requires:       python38-scipy
Requires:       python38-setuptools
Requires:       python38-tvm = %{version}
Requires:       python38-typed-ast
Recommends:     python38-Pillow
Recommends:     python38-onnx
%else
Requires:       python3-scipy
Requires:       python3-setuptools
Requires:       python3-tvm = %{version}
Requires:       python3-typed-ast
Recommends:     python3-Pillow
Recommends:     python3-onnx
%endif

%description -n tvmc
TVMC is a tool that exposes TVM features such as auto-tuning, compiling,
profiling and execution of models, via a command line interface.

%package -n %{name}-devel
Summary:        Headers for the TVM Deep Learning Compiler Stack
Requires:       libtvm = %{version}

%description -n %{name}-devel
TVM is a deep learning compiler stack for CPUs, GPUs, and specialized accelerators.

This package contains the headers.

%package -n libtvm
Summary:        Libraries generated for TVM
# renamed up to libtvm here
Provides:       tvm = %{version}-%{release}
Obsoletes:      tvm < %{version}-%{release}

%description -n libtvm
Libraries generated for TVM without any provided soname.

%prep
%setup -q
%autopatch -p1

# Workaround - https://discuss.tvm.ai/t/build-fails-on-tvm-0-6-0-0-6-1-with-gcc10-and-gcc7/7462/5?u=ggardet
ln -s %{_includedir}/endian.h include/endian.h

%build
%limit_build -m 800
# USE_CUDA - we would need cuda
# USE_METAL
# USE_MICRO USE_MICRO_STANDALONE_RUNTIME
# USE_NNPACK
# USE_ROCBLAS USE_ROCM
%cmake \
%if %{with arm_compute_lib}
  -DUSE_ARM_COMPUTE_LIB_GRAPH_RUNTIME=ON \
  -DUSE_ARM_COMPUTE_LIB=ON \
%endif
  -DDMLC_PATH="%{_includedir}/dmlc" \
  -DDLPACK_PATH="%{_includedir}/dlpack" \
  -DRANG_PATH="%{_includedir}/rang" \
  -DUSE_GRAPH_RUNTIME=ON \
  -DUSE_LLVM=ON \
  -DUSE_BLAS="openblas" \
%if %{with onednn}
  -DUSE_MKLDNN=ON \
%else
  -DUSE_MKLDNN=OFF \
%endif
  -DUSE_OPENCL=ON \
  -DUSE_OPENGL=ON \
  -DUSE_OPENMP=ON \
  -DUSE_RANDOM=ON \
  -DUSE_RPC=ON \
  -DUSE_RTTI=ON \
  -DUSE_SORT=ON \
  -DUSE_THREADS=ON \
  -DUSE_VULKAN=ON \
  -DUSE_LIBBACKTRACE=OFF \
  -DUSE_ANTLR="/usr/share/java/antlr4/antlr4-runtime.jar" \
  -DINSTALL_DEV=ON
%cmake_build
cd ..
export TVM_LIBRARY_PATH="$(pwd)/%{__builddir}"
pushd python
# Fix rpm runtime dependency rpmlint error replace the shebang in all the scripts with %%{_bindir}/python3
find . -name "*.py" -exec sed -i 's|#!%{_bindir}/env python|#!%{_bindir}/python3|' {} ";"
%python_build
popd

%install
%cmake_install
# remove endian hack
rm -f %{buildroot}%{_includedir}/endian.h
export TVM_LIBRARY_PATH="$(pwd)/%{__builddir}"
pushd python
%python_install
%python_expand chmod 0755 %{buildroot}%{$python_sitearch}/tvm/driver/tvmc/main.py %{buildroot}%{$python_sitearch}/tvm/contrib/torch/pytorch_tvm.py
popd
# Remove /usr/tvm/*.so
rm -rf %{buildroot}%{_prefix}/tvm
# Remove .cpp file
%python_expand rm %{buildroot}/%{$python_sitearch}/tvm/_ffi/_cython/core.cpp
%python_expand %fdupes %{buildroot}%{$python_sitearch}

%check
pushd %{__builddir}
%cmake_build cpptest
popd
export LD_LIBRARY_PATH=%{buildroot}%{_libdir}:$(pwd)
# to avoid CI thread throttling.
export TVM_BIND_THREADS=0
export OMP_NUM_THREADS=1
%{python_expand #
export PYTHONPATH=%{buildroot}%{$python_sitearch}
export PYTHONDONTWRITEBYTECODE=1
# TextureCopy: no openCL in environment
ctestflags="-E TextureCopy"
%ctest $ctestflags
}

%if %{with pytest}
mkdir python_gen
cp python/gen_requirements.py python_gen/
export PYTHONPATH=$(pwd)/python_gen
export TVM_INCLUDE_PATH=%{buildroot}%{_prefix}
export TVM_TEST_TARGETS="llvm"
export TVM_FFI="cython"
# No python module for XGBoost
donttest="test_xgb_model"
donttest="$donttest or test_sketch_search_policy_xgbmodel"
donttest="$donttest or test_sketch_search_policy_custom_sketch"
donttest="$donttest or test_task_tuner_without_measurement"
donttest="$donttest or test_autotvm_xgboost_mode"
# No OpenCL device
donttest="$donttest or test_simplex_data_transferring"
donttest="$donttest or test_duplex_data_transferring"
# no minrpc in installed path (test looks in src)
donttest="$donttest or test_rpc_echo"
donttest="$donttest or test_rpc_remote_module"
%ifnarch x86_64
# test_fp16_to_fp32 fails on non-x86 as it uses skylake as llvm target
donttest="$donttest or test_fp16_to_fp32"
%endif
%ifarch ppc64le
donttest="$donttest or test_popcount or test_vmlal_s16 or test_llvm_add_pipeline"
%endif
%ifarch ppc64
donttest="$donttest or test_check_correctness or test_graph_simple or test_llvm_add_pipeline or test_popcount or test_rpc_array or test_rpc_file_exchange or test_rpc_remote_module or test_rpc_return_func or test_rpc_return_ndarray or test_rpc_simple or test_rpc_tracker_register or test_rpc_tracker_request or test_vmlal_s16"
%endif
# probes vulkan on test collection
ignorefiles="--ignore tests/python/unittest/test_target_codegen_vulkan.py"
ignorefiles="$ignorefiles --ignore tests/python/unittest/test_tir_intrin.py"
%if 0%{?suse_version} <= 1500
# Skip some tests on Leap/SLE (some tests would need python 3.7+)
donttest="$donttest or test_meta_schedule_local_runner_time_out or test_meta_schedule_local_runner_exception"
%endif
%pytest_arch -v tests/python/unittest -m "not gpu" -k "not ($donttest)" $ignorefiles
%endif

%post -n libtvm -p /sbin/ldconfig
%postun -n libtvm -p /sbin/ldconfig

%files -n tvmc
%{_bindir}/tvmc

%files -n libtvm
%license LICENSE
%doc README.md
%{_libdir}/libtvm*.so

%files -n %{name}-devel
%{_includedir}/tvm

%files %{python_files}
%{python_sitearch}/tvm
%{python_sitearch}/tvm-%{version}*-info

%changelog
* Tue Nov 22 2022 Max Lin <mlin@suse.com>
- Build against llvm14 for Leap 15.5 since llvm15 is the default llvm
  version in Leap 15.5 which tvm is not supported yet
* Fri Oct 14 2022 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Build against llvm14 for Tumbleweed since llvm15 is not
  supported yet
* Tue Jun 28 2022 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Requires python38-* for tvmc on Tumbleweed since the default
  python (3.10) is not compatible yet, so no python3-* available
* Wed Apr  6 2022 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Add patch to fix boo#1197347:
  * tvm-do-not-force-synr-version.patch
* Mon Mar  7 2022 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Skip test_meta_schedule_local_runner_time_out and
  test_meta_schedule_local_runner_exception tests on Leap/SLE
  since they require python 3.7+
* Wed Mar  2 2022 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Add deps on python3-tvm and python3-setuptools for tvmc - boo#1196646
* Tue Mar  1 2022 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Add 'Requires: python-cloudpickle' - boo#1195952#c1
* Sun Feb 27 2022 Ben Greiner <code@bnavigator.de>
- Update to v0.8
  * PaddlePaddle frontend
  * TVMScript: round-trippable python-based syntax for TIR
  * TorchScript integration
  * TensorIR scheduling language
  * TensorRT and CUTLASS integration via BYOC
  * Int4 TensorCore support in AutoTVM
  * MicroTVM Project API and Zephyr, Arduino support
  * AOT executor
  * Robust Windows support
  * Affine analysis infra: iter-affine-map
  * Improved Vulkan backend
  * CUDA graph support in TVM runtime
- Hopefully fixes boo#1195952
- Drop Patches merged upstream:
  * tvm-fix-catch.patch -- gh#apache/tvm#7319
  * tvm-fix-llvm12.patch -- gh#apache/tvm#6717, gh#apache/tvm#6738
- Add patches
  * tvm-fix-relay-test.patch -- gh#apache/tvm#10402
  * tvm-disable-vulkan-test-check.patch
- Skip Python 3.9+ -- gh#apache/tvm#8577
* Mon Nov  8 2021 Jan Engelhardt <jengelh@inai.de>
- Trim future plans/marketing from description.
* Tue Oct 19 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Build fails with LLVM 13, so still use LLVM 12 on Tumbleweed
* Fri Apr 30 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Backport patches to fix build on Tumbleweed with llvm12:
  * tvm-fix-llvm12.patch
* Tue Feb 16 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- NEP 29: Last minorversion bump deprecated Python 3.6
  https://numpy.org/neps/nep-0029-deprecation_policy.html
* Wed Jan 27 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Recommends onnx and PIL for tvmc
* Tue Jan 26 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Fix upstream tarball
* Tue Jan 26 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Disable LTO as it breaks runtime, see: https://github.com/apache/tvm/issues/6832
- Disable python tests as it does not find Vulkan/openCL
* Thu Jan 21 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Add patch to fix a warning with gcc10:
  * tvm-fix-catch.patch
* Wed Jan 20 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Add Arm Compute Library support
* Wed Oct 28 2020 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update to 0.7.0: https://github.com/apache/incubator-tvm/blob/main/NEWS.md
  v0.7 brings many major features. The community works together
  to refactor the internal code base to bring an unified IR code
  structure with a unified IRModule, type system and pass
  infrastructure. We have also bought many exciting new features,
  some highlights include:
  * Initial automatic scheduling support
  * Initial command line driver interface
  * WebGPU and webassembly support
  * Better first class rust support in the codebase
  * Intial Hexagon support
  * Bring your own codegen (BYOC) support
  The community also continues to bring high quality improvements
  to the existing modules including, but not limited to:
  better frontend coverage, performance, quantization, uTVM
  and dynamic shape support.
- Rebase pacth:
  * tvm-fix-openblas.patch
- Drop patch:
  * lib-finder-python-cmake.patch
* Fri Oct  2 2020 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Enable onednn (formerly mkl-dnn) for aarch64 and ppc64le as well
- Add support to:
  * Antlr4
  * openBLAS
- Add patch to fix build with openblas enabled:
  * tvm-fix-openblas.patch
* Fri Oct  2 2020 Michel Normand <normand@linux.vnet.ibm.com>
- Add _constraints with 4GB min disk space
- Use limit_build macro for all arches
- Exclude more tests for ppc64 & ppc64le
* Wed Sep  9 2020 Tomáš Chvátal <tchvatal@suse.com>
- Fix typo
* Tue Sep  8 2020 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Exclude %%arm %%{ix86} as tests fail
* Tue Sep  8 2020 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Skip tests requiring opencl
- Skip test requiring a specific arch
* Tue Sep  8 2020 Tomáš Chvátal <tchvatal@suse.com>
- Split cmake folder to not override it with python builds
- Fix detection of this folder in the python modules:
  * lib-finder-python-cmake.patch
* Tue Sep  8 2020 Tomáš Chvátal <tchvatal@suse.com>
- Add provides/obsoletes on TVM
* Mon Sep  7 2020 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Build fails on Tumbleweed with llvm10, so use llvm9 - boo#1176220
* Mon Sep  7 2020 Tomáš Chvátal <tchvatal@suse.com>
- Format with spec-cleaner
- Expand properly the fdupes call
- Fix headers install
- Fix python pkg names
- Fix python requires
* Thu Sep  3 2020 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Package nnvm and topi python packages
* Thu Sep  3 2020 Guillaume GARDET <guillaume.gardet@opensuse.org>
- mkl-dnn is not available on %%{ix86}, so do not enable it there
* Thu Sep  3 2020 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Make use of python Macros: %%python_subpackages and %%python_files
* Wed Sep  2 2020 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Enable Python binding
* Wed Sep  2 2020 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Enable UnitTests (only when LLVM is enabled as it is a requirement)
* Tue Sep  1 2020 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Fix build
* Thu Jul 30 2020 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Dep is dlpack-devel, not dlpack
* Wed Jul 29 2020 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Disable mkldnn on non-x86
- Add dep to dlpack
- Update to 0.6.1
* Mon Jun  1 2020 Tomáš Chvátal <tchvatal@suse.com>
- Initial commit, needed by mxnet
